#!/usr/bin/env python
# -*- coding: latin-1 -*-
# pragma: no-cover

__title__ = 'towers'
__description__ = 'Towers of Hanoi for Python'
__url__ = 'http://towers.readthedocs.io/en/latest'
__version__ = '0.1.1'
__author__ = 'Francis Horsman'
__author_email__ = 'francis.horsman@gmail.com'
__license__ = 'MIT'
__copyright__ = 'Copyright 2017 Francis Horsman'
